<?php

App::uses('ModelBehavior', 'Model');
App::uses('ModelValidator', 'Model');
App::uses('Session', 'Model/Datasource');

class CustomFieldsDataBehavior extends ModelBehavior
{
    /**
     * save the error state incase it somehow
     * gets through and goes to afterSave
     */
    public $validationErrors = array();

    /**
     * validate the custom fields and die out if needed
     */
    /**
     * beforeSave is called before a model is saved. Returning false from a beforeSave callback
     * will abort the save operation.
     *
     * @param Model $model Model using this behavior
     * @param array $options Options passed from Model::save().
     * @return mixed False if the operation should abort. Any other result will continue.
     * @see Model::save()
     */
    public function beforeSave(Model $Model, $options = array())
    {
        $return = parent::beforeSave($Model, $options);

        if (! empty($Model->data[$Model->alias]['CustomFieldData'])) {

            $CustomFieldData = EvClassRegistry::init('EvCustomFields.CustomFieldData');
            $FieldData = EvClassRegistry::init('EvCustomFields.FieldData');
            $Field = EvClassRegistry::init('EvCustomFields.Field');

            $validationRules = $Field->getValidation(array_keys($Model->data[$Model->alias]['CustomFieldData']));

            foreach ($Model->data[$Model->alias]['CustomFieldData'] as $fieldId => $field) {

                $compiledRules = $this->compileCFValidationRules($validationRules, $fieldId);

                $validates = $this->validateCustomField(
                    $FieldData,
                    $Model->data[$Model->alias]['CustomFieldData'][$fieldId]['FieldData']['value'],
                    $compiledRules
                );

                if (! $validates) {
                    $error = true;
                    $CustomFieldData->validationErrors[$fieldId]['FieldData']['value'] = $FieldData->validationErrors['value'];
                }
            }
        }
/*
        if ($error) {
            CakeSession::write('cf_validationErrors', $CustomFieldData->validationErrors);
        }*/

        return (
            $return &&
            (! isset($error) || $error === false)
        );
    }


    /**
     * Save the custom field data
     *
     * @param Model $model Model using this behavior
     * @param bool $created True if this save created a new record
     * @param array $options Options passed from Model::save().
     * @return bool
     * @see Model::save()
     */
    public function afterSave(Model $Model, $created, $options = array())
    {
        $return = parent::afterSave($Model, $created, $options);
        $result = true;

        if (! empty($Model->data[$Model->alias]['CustomFieldData'])) {

            $FieldData = EvClassRegistry::init('EvCustomFields.FieldData');

            // setup the model / model id
            $modelInfo = array(
                'model' => EvClassRegistry::getNameFromModel($Model),
                'model_id' => $Model->data[$Model->alias][$Model->primaryKey]
            );

            foreach ($Model->data[$Model->alias]['CustomFieldData'] as $fieldId => $field) {

                $Model->data[$Model->alias]['CustomFieldData'][$fieldId]['FieldData']['field_id'] = $fieldId;
                $Model->data[$Model->alias]['CustomFieldData'][$fieldId]['FieldData']['model'] = $modelInfo['model'];
                $Model->data[$Model->alias]['CustomFieldData'][$fieldId]['FieldData']['model_id'] = $modelInfo['model_id'];
            }

            $result = $FieldData->saveMany(
                $Model->data[$Model->alias]['CustomFieldData']
            );
        }

        return ($return && $result);
    }


    /**
     * validate the custom field data
     *
     * @param   Object  FieldData Model
     * @param   mixed   The actual form value
     * @param   array   Compiled array of rules to validate on
     * @return  bool
     */
    public function validateCustomField($FieldData, $value, $rules)
    {
        $FieldData->clear();
        $FieldData->set('value', $value);

        $Validator = new ModelValidator($FieldData);
        $Validator->add(
            'value',
            $rules
        );

        return $Validator->validates();
    }


    /**
     * compile the validation rules
     *
     * @param   array   Array of all the vlaidation rules for all custom fields we are saving
     * @param   int     Field id of the current field we are saving
     * @return  array   Compiled array of validation rules to apply to field
     */
    public function compileCFValidationRules($validation, $fieldId)
    {
        $compiledRules = array();

        foreach ($validation[$fieldId] as $rule) {

            if (empty($rule['rule'])) {
                continue;
            }

            $ruleType = $rule['rule'];
            if (! empty($rule['values'])) {
                $values = explode(',', $rule['values']);
                $ruleType = array_merge(array($rule['rule']), $values);
            }

            $compiledRules[$rule['rule']] = array(
                'rule' => $ruleType,
                'message' => $rule['message']
            );
        }

        return $compiledRules;
    }


    /**
     * define an afterFind to retrieve hte custom field data for the item
     *
     * @param Model $model Model using this behavior
     * @param mixed $results The results of the find operation
     * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
     * @return mixed An array value will replace the value of $results - any other value will be ignored.
     */
    public function afterFind(Model $Model, $results, $primary = false)
    {
        // get all the model ids
        $modelIds = Hash::extract($results, '{n}.' . $Model->alias . '.' . $Model->primaryKey);

        if (empty($modelIds)) {
            return $results;
        }

        // get all the fields and data where id IN model_id
        $FieldData = EvClassRegistry::init('EvCustomFields.FieldData');
        $customData = $FieldData->find(
            'all',
            array(
                'contain' => 'Field',
                'conditions' => array(
                    'FieldData.model' => EvClassRegistry::getNameFromModel($Model),
                    'FieldData.model_id' => $modelIds
                )
            )
        );

        if (! empty($customData)) {

            $customData = Hash::combine(
                $customData,
                '{n}.FieldData.field_id',
                '{n}',
                '{n}.FieldData.model_id'
            );

            // loop the results and assign the field data to the result
            foreach ($results as $key => $item) {

                if (empty($item[$Model->alias][$Model->primaryKey])) {
                    continue;
                }

                if (! empty($customData[$item[$Model->alias][$Model->primaryKey]])) {

                    foreach ($customData[$item[$Model->alias][$Model->primaryKey]] as $field_id => $data) {

                        $results[$key]['CustomFieldData'][$field_id] = $data;
                    }
                }
            }
        }

/*        if (CakeSession::check('cf_validationErrors')) {
            debug(debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 20));
            debug(CakeSession::read('cf_validationErrors'));

            $CustomFieldData = EvClassRegistry::init('EvCustomFields.CustomFieldData');
            $CustomFieldData->validationErrors = CakeSession::read('cf_validationErrors');
            CakeSession::delete('cf_validationErrors');
        }*/

        return $results;
    }
}
